##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: __init__.py 97 2007-03-29 22:58:27Z rineichen $
"""

import doctest
import unittest
import zope.interface
import zope.component
from zope.traversing.browser.interfaces import IAbsoluteURL
from zope.traversing.interfaces import IPhysicallyLocatable
from zope.site import hooks
from zope.app.testing import setup

import z3c.testing
from z3c.menu.ready2go import interfaces
from z3c.menu.ready2go import item
from z3c.menu.ready2go import manager
from z3c.menu.ready2go import testing


class CheckerStub(object):
    """Just a checker stub."""

    def __init__(self, context, request, view, menu, item):
        self.context = context
        self.request = request
        self.view = view
        self.menu = menu
        self.item = item

    @property
    def available(self):
        return True

    @property
    def selected(self):
        return True


class ParentStub(object):
    """Just an object supporting a context attribtute."""

    __name__ = __parent__ = context = None

    zope.interface.implements(IPhysicallyLocatable)

    def __init__(self, path=('a', 'b')):
        self.path = path

    def getRoot(self):
        return self

    def getPath(self):
        return self.path


class AbsoulteURLStub(object):
    """Absolute url stub."""

    def __init__(self, context, request):
        pass

    def __str__(self):
        return u'here'

    __call__ = __str__


class MenuManagerTest(z3c.testing.InterfaceBaseTest):

    def getTestInterface(self):
        return interfaces.IMenuManager

    def getTestClass(self):
        return manager.MenuManager

    def getTestPos(self):
        return (None, None, None)


class GlobalMenuItemTest(z3c.testing.InterfaceBaseTest):

    def setUp(self):
        site = setup.placefulSetUp(site=True)
        hooks.setSite(site)
        zope.component.provideAdapter(AbsoulteURLStub, (None, None),
            IAbsoluteURL)
        zope.component.provideAdapter(CheckerStub, (None, None, None, None,
            None), interfaces.ISelectedChecker)
        super(GlobalMenuItemTest, self).setUp()

    def getTestInterface(self):
        return interfaces.IGlobalMenuItem

    def getTestClass(self):
        return item.GlobalMenuItem

    def getTestPos(self):
        return (ParentStub(), None, ParentStub(), None)


class SiteMenuItemTest(z3c.testing.InterfaceBaseTest):

    def setUp(self):
        site = setup.placefulSetUp(site=True)
        hooks.setSite(site)
        zope.component.provideAdapter(AbsoulteURLStub, (None, None),
            IAbsoluteURL)
        zope.component.provideAdapter(CheckerStub, (None, None, None, None,
            None), interfaces.ISelectedChecker)
        super(SiteMenuItemTest, self).setUp()

    def getTestInterface(self):
        return interfaces.ISiteMenuItem

    def getTestClass(self):
        return item.SiteMenuItem

    def getTestPos(self):
        return (ParentStub(), None, ParentStub(), None)


class ContextMenuItemTest(z3c.testing.InterfaceBaseTest):

    def setUp(self):
        zope.component.provideAdapter(CheckerStub, (None, None, None, None,
            None), interfaces.ISelectedChecker)
        super(ContextMenuItemTest, self).setUp()

    def getTestInterface(self):
        return interfaces.IContextMenuItem

    def getTestClass(self):
        return item.ContextMenuItem

    def getTestPos(self):
        return (None, None, ParentStub(), None)


def test_suite():
    return unittest.TestSuite((
        doctest.DocFileSuite('README.txt',
            setUp=testing.setUp, tearDown=testing.tearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            ),
        doctest.DocFileSuite('zcml.txt',
            setUp=testing.setUp, tearDown=testing.tearDown,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
            ),
        unittest.makeSuite(MenuManagerTest),
        unittest.makeSuite(GlobalMenuItemTest),
        unittest.makeSuite(SiteMenuItemTest),
        unittest.makeSuite(ContextMenuItemTest),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: tests.py 82943 2008-01-18 10:01:06Z rogerineichen $
"""
__docformat__ = 'restructuredtext'

import zope.security
from zope.publisher.interfaces.browser import IBrowserView
from zope.app.testing import setup
from zope.app.testing import ztapi
from zope.container import contained

from z3c.menu.ready2go import interfaces
from z3c.menu.ready2go import item


class TestParticipation(object):
    principal = 'foobar'
    interaction = None


class ISample(zope.interface.Interface):
    """Sample context interface."""


class Sample(object):
    """Sample context object."""

    zope.interface.implements(ISample)

    def __init__(self, title):
        self.title = title


class LocatableView(contained.Contained):

    zope.interface.implements(IBrowserView)

    def __init__(self, context, request):
        self.__parent__ = context
        self.context = context
        self.request = request

class IFirstView(IBrowserView):
    """First sample view interface."""

class ISecondView(IBrowserView):
    """Second sample view interface."""

class FirstView(LocatableView):
    """First view."""

    zope.interface.implements(IFirstView)

class SecondView(LocatableView):
    """Second view."""

    zope.interface.implements(ISecondView)


class IFirstMenu(interfaces.IMenuManager):
    """First menu manager."""

class ISecondMenu(interfaces.IMenuManager):
    """Second menu manager."""


class FirstMenuItem(item.ContextMenuItem):
    viewName = 'first.html'

class SecondMenuItem(item.ContextMenuItem):
    viewName = 'second.html'


def setUp(test):
    root = setup.placefulSetUp(site=True)
    test.globs['root'] = root

    from zope.browserpage import metaconfigure
    from zope.contentprovider import tales
    metaconfigure.registerType('provider', tales.TALESProviderExpression)

    zope.security.management.getInteraction().add(TestParticipation())


def tearDown(test):
    setup.placefulTearDown()



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: layer.py 197 2007-04-13 05:03:32Z rineichen $
"""

import zope.interface
import zope.component
from zope.publisher.interfaces.browser import IBrowserRequest

from z3c.menu.ready2go import interfaces


class CheckerBase(object):
    """Generic checker base class."""

    def __init__(self, context, request, view, menu, item):
        self.context = context
        self.request = request
        self.view = view
        self.menu = menu
        self.item = item


# ISelectedChecker
class FalseSelectedChecker(CheckerBase):
    """False selected checker can avoid selected menu item rendering."""

    zope.interface.implements(interfaces.ISelectedChecker)

    @property
    def selected(self):
        return False


class TrueSelectedChecker(CheckerBase):
    """True selected checker can force selected menu item rendering."""

    zope.interface.implements(interfaces.ISelectedChecker)

    @property
    def selected(self):
        return True


class ViewNameSelectedChecker(CheckerBase):
    """Selected by view name offers a generic checker for IContextMenuItem."""

    zope.interface.implements(interfaces.ISelectedChecker)

    @property
    def selected(self):
        """Selected if also view name compares."""
        viewName = self.item.viewName
        if viewName.startswith('@@'):
            viewName = viewName[2:]
        if self.view.__name__ == viewName:
            return True
        return False


# default selected checkers
class GlobalSelectedChecker(FalseSelectedChecker):
    """Global menu item selected checker.
    
    Note, this is a menu group which is selected on different menu items.
    You need to register for each view a TrueSelectedChecker if the site menu
    item should get rendered as selected.
    """

    zope.component.adapts(zope.interface.Interface, IBrowserRequest,
        zope.interface.Interface, interfaces.IMenuManager,
        interfaces.IGlobalMenuItem)


class SiteSelectedChecker(FalseSelectedChecker):
    """Site menu item selected checker.
    
    Note, this is a menu group which is selected on different menu items.
    You need to register for each view a TrueSelectedChecker if the site menu
    item should get rendered as selected.
    """

    zope.component.adapts(zope.interface.Interface, IBrowserRequest,
        zope.interface.Interface, interfaces.IMenuManager,
        interfaces.ISiteMenuItem)


class ContextSelectedChecker(ViewNameSelectedChecker):
    """Context menu item selected checker."""

    zope.component.adapts(zope.interface.Interface, IBrowserRequest,
        zope.interface.Interface, interfaces.IMenuManager,
        interfaces.IContextMenuItem)






##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id:$
"""
__docformat__ = "reStructuredText"

import zope.interface
import zope.configuration.fields
import zope.security.checker
from zope.component import zcml
from zope.configuration.exceptions import ConfigurationError
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.publisher.interfaces.browser import IBrowserView
from zope.viewlet import viewlet
from zope.viewlet.metadirectives import IViewletDirective

from zope.browserpage.metaconfigure import _handle_permission
from zope.browserpage.metaconfigure import _handle_allowed_attributes
from zope.browserpage.metaconfigure import _handle_for
from zope.browserpage.metaconfigure import _handle_allowed_interface

from z3c.menu.ready2go import checker
from z3c.menu.ready2go import interfaces
from z3c.menu.ready2go import item
from z3c.menu.ready2go.i18n import MessageFactory as _


class IMenuSelectorDirective(zope.interface.Interface):
    """A directive to register a menu selector."""

    factory = zope.configuration.fields.GlobalObject(
        title=_("Selector factory"),
        description=_("Python name of a factory which can create the"
                      " selector object.  This must identify an"
                      " object in a module using the full dotted name."),
        required=False,
        default=checker.TrueSelectedChecker)

    for_ = zope.configuration.fields.GlobalObject(
        title=u"Context",
        description=u"The content interface or class this selector is for.",
        required=False)

    view = zope.configuration.fields.GlobalObject(
        title=_("The view the selector is registered for."),
        description=_("The view can either be an interface or a class. By "
                      "default the provider is registered for all views, "
                      "the most common case."),
        required=False,
        default=IBrowserView)

    layer = zope.configuration.fields.GlobalObject(
        title=_("The layer the view is in."),
        description=_("""
        A skin is composed of layers. It is common to put skin
        specific views in a layer named after the skin. If the 'layer'
        attribute is not supplied, it defaults to 'default'."""),
        required=False,
        default=IBrowserRequest)

    manager = zope.configuration.fields.GlobalObject(
        title=u"Menu Manager",
        description=u"The menu manager interface or class this selector is for.",
        required=False,
        default=interfaces.IMenuManager)

    menu = zope.configuration.fields.GlobalObject(
        title=u"Menu Item",
        description=u"The menu item interface or class this selector is for.",
        required=False,
        default=interfaces.IMenuItem)


class IMenuItemDirective(IViewletDirective):
    """Menu item directive."""

    title = zope.configuration.fields.MessageID(
        title=u"I18n title",
        description=u"Translatable title for a viewlet.",
        required=False)

# Arbitrary keys and values are allowed to be passed to the menu item.
IMenuItemDirective.setTaggedValue('keyword_arguments', True)


# menuItem directive
def menuItemDirective(
    _context, name, permission, for_=zope.interface.Interface,
    layer=IDefaultBrowserLayer, view=IBrowserView,
    manager=interfaces.IMenuManager, class_=None, template=None,
    attribute='render', allowed_interface=None, allowed_attributes=None,
    title=None, **kwargs):

    # Security map dictionary
    required = {}

    if title is not None:
        # set i18n aware title
        kwargs['i18nTitle'] = title

    # Get the permission; mainly to correctly handle CheckerPublic.
    permission = _handle_permission(_context, permission)

    # Either the class or template must be specified.
    if not (class_ or template):
        raise ConfigurationError("Must specify a class or template")

    # Make sure that all the non-default attribute specifications are correct.
    if attribute != 'render':
        if template:
            raise ConfigurationError(
                "Attribute and template cannot be used together.")

        # Note: The previous logic forbids this condition to evere occur.
        if not class_:
            raise ConfigurationError(
                "A class must be provided if attribute is used")

    # Make sure that the template exists and that all low-level API methods
    # have the right permission.
    if template:
        template = os.path.abspath(str(_context.path(template)))
        if not os.path.isfile(template):
            raise ConfigurationError("No such file", template)
        required['__getitem__'] = permission

    # Make sure the has the right form, if specified.
    if class_:
        if attribute != 'render':
            if not hasattr(class_, attribute):
                raise ConfigurationError(
                    "The provided class doesn't have the specified attribute "
                    )
        if template:
            # Create a new class for the viewlet template and class.
            new_class = viewlet.SimpleViewletClass(
                template, bases=(class_, ), attributes=kwargs, name=name)
        else:
            if not hasattr(class_, 'browserDefault'):
                cdict = {'browserDefault':
                         lambda self, request: (getattr(self, attribute), ())}
            else:
                cdict = {}

            cdict['__name__'] = name
            cdict['__page_attribute__'] = attribute
            cdict.update(kwargs)
            new_class = type(class_.__name__,
                             (class_, viewlet.SimpleAttributeViewlet), cdict)

        if hasattr(class_, '__implements__'):
            zope.interface.classImplements(new_class, IBrowserPublisher)

    else:
        # Create a new class for the viewlet template alone.
        new_class = viewlet.SimpleViewletClass(template, name=name,
                                               attributes=kwargs)

    # Set up permission mapping for various accessible attributes
    _handle_allowed_interface(
        _context, allowed_interface, permission, required)
    _handle_allowed_attributes(
        _context, allowed_attributes, permission, required)
    _handle_allowed_attributes(
        _context, kwargs.keys(), permission, required)
    _handle_allowed_attributes(
        _context,
        (attribute, 'browserDefault', 'update', 'render', 'publishTraverse'),
        permission, required)

    # Register the interfaces.
    _handle_for(_context, for_)
    zcml.interface(_context, view)

    # Create the security checker for the new class
    zope.security.checker.defineChecker(new_class,
        zope.security.checker.Checker(required))

    # register viewlet
    _context.action(
        discriminator = ('viewlet', for_, layer, view, manager, name),
        callable = zcml.handler,
        args = ('registerAdapter',
                new_class, (for_, layer, view, manager),
                zope.viewlet.interfaces.IViewlet, name, _context.info),)


def addMenuItemDirective(_context, name, permission,
    for_=zope.interface.Interface, layer=IDefaultBrowserLayer,
    view=IBrowserView, manager=interfaces.IMenuManager,
    class_=item.AddMenuItem, template=None, attribute='render',
    allowed_interface=None, allowed_attributes=None, title=None, **kwargs):
    menuItemDirective(_context, name, permission, for_, layer, view, manager,
        class_, template, attribute, allowed_interface, allowed_attributes,
        title, **kwargs)


def contextMenuItemDirective(_context, name, permission,
    for_=zope.interface.Interface, layer=IDefaultBrowserLayer,
    view=IBrowserView, manager=interfaces.IMenuManager,
    class_=item.ContextMenuItem,template=None, attribute='render',
    allowed_interface=None, allowed_attributes=None, title=None, **kwargs):
    menuItemDirective(_context, name, permission, for_, layer, view, manager,
        class_, template, attribute, allowed_interface, allowed_attributes,
        title, **kwargs)


def globalMenuItemDirective(_context, name, permission,
    for_=zope.interface.Interface, layer=IDefaultBrowserLayer,
    view=IBrowserView, manager=interfaces.IMenuManager,
    class_=item.GlobalMenuItem, template=None, attribute='render',
    allowed_interface=None, allowed_attributes=None, title=None, **kwargs):
    menuItemDirective(_context, name, permission, for_, layer, view, manager,
        class_, template, attribute, allowed_interface, allowed_attributes,
        title, **kwargs)


def siteMenuItemDirective(_context, name, permission,
    for_=zope.interface.Interface, layer=IDefaultBrowserLayer,
    view=IBrowserView, manager=interfaces.IMenuManager,
    class_=item.SiteMenuItem, template=None, attribute='render',
    allowed_interface=None, allowed_attributes=None, title=None, **kwargs):
    menuItemDirective(_context, name, permission, for_, layer, view, manager,
        class_, template, attribute, allowed_interface, allowed_attributes,
        title, **kwargs)


# menu selector directive
def menuSelectorDirective(
    _context, factory=checker.TrueSelectedChecker,
    for_=zope.interface.Interface, layer=IBrowserRequest, view=IBrowserView,
    manager=interfaces.IMenuManager, menu=interfaces.IMenuItem):

    # Security map dictionary
    objs = (for_, layer, view, manager ,menu)
    factory = (factory,)

    zcml.adapter(_context, factory, provides=interfaces.ISelectedChecker,
        for_=objs, permission=None, name='', trusted=False, locate=False)



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: layer.py 197 2007-04-13 05:03:32Z rineichen $
"""

from z3c.menu.ready2go import interfaces


class IGlobalMenu(interfaces.IMenuManager):
    """CSS viewlet manager."""


class ISiteMenu(interfaces.IMenuManager):
    """CSS viewlet manager."""


class IContextMenu(interfaces.IMenuManager):
    """CSS viewlet manager."""


class IAddMenu(interfaces.IMenuManager):
    """CSS viewlet manager."""



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: layer.py 197 2007-04-13 05:03:32Z rineichen $
"""

from z3c.menu.ready2go.i18n import MessageFactory as _
import zope.interface
import zope.schema
import zope.viewlet.interfaces


class IMenuManager(zope.viewlet.interfaces.IViewletManager):
    """Generic nenu manager."""

    def render():
        """Represent the menu"""


class IMenuItem(zope.viewlet.interfaces.IViewlet):
    """Menu item base."""

    template = zope.interface.Attribute("""Page template""")

    contextInterface = zope.interface.Attribute(
        """Context discriminator interface""")

    viewInterface = zope.interface.Attribute(
        """View discriminator interface""")

    title = zope.schema.TextLine(
        title=_('Title'),
        description=_('Menu item title'),
        default=u''
        )

    viewName = zope.schema.TextLine(
        title=_('View name'),
        description=_('Name of the view which the menu points to.'),
        default=u''
        )

    weight = zope.schema.TextLine(
        title=_('Weight'),
        description=_('Weight of the menu item order.'),
        default=u''
        )

    cssActive = zope.schema.TextLine(
        title=_('Active CSS class name'),
        description=_('CSS class name for active menu items'),
        default=u''
        )

    cssInActive = zope.schema.TextLine(
        title=_('In-Active CSS class name'),
        description=_('CSS class name for inactive menu items'),
        default=u''
        )

    css = zope.schema.TextLine(
        title=_('CSS class name'),
        description=_('CSS class name'),
        default=u''
        )

    available = zope.schema.Bool(
        title=_('Available'),
        description=_('Marker for available menu item'),
        default=True
        )

    selected = zope.schema.Bool(
        title=_('Selected'),
        description=_('Marker for selected menu item'),
        default=False
        )

    url = zope.schema.TextLine(
        title=_('URL'),
        description=_('URL or other url like javascript function.'),
        default=u''
        )

    subMenuProviderName = zope.schema.TextLine(
        title=_('Sub menu provider name'),
        description=_('Name of the sub menu provider.'),
        default=u''
        )

    def getURLContext():
        """Returns the context the base url."""

    def render():
        """Return the template with the option 'menus'"""


class ISelectedChecker(zope.interface.Interface):
    """Selected checker."""

    selected = zope.schema.Bool(
        title=_('Selected'),
        description=_('Marker for selected menu item'),
        default=False
        )


class IGlobalMenuItem(IMenuItem):
    """Menu item with ZODB application root as url base."""


class ISiteMenuItem(IMenuItem):
    """Menu item with nearest site as url base."""


class IContextMenuItem(IMenuItem):
    """Menu item with context as url base."""


class IAddMenuItem(IMenuItem):
    """Add menu item with context as url base."""



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: layer.py 197 2007-04-13 05:03:32Z rineichen $
"""
__docformat__ = "reStructuredText"

import zope.interface
import zope.proxy
from zope.traversing.api import getRoot
from zope.traversing.browser import absoluteURL

from zope.viewlet import viewlet
from zope.site import hooks

from z3c.template.template import getPageTemplate
from z3c.menu.ready2go import interfaces


# base menu item mixin
class MenuItem(viewlet.ViewletBase):
    """Menu item base."""

    template = getPageTemplate()

    # see z3c:add/context/site/globalMenuItemDirective
    i18nTitle = None

    # internal approved values
    approved = False
    approvedURL = None

    # url view name if different then ``selected`` viewName
    viewName = u'index.html'

    # ``selected`` discriminator values
    contextInterface = zope.interface.Interface
    viewInterface = zope.interface.Interface
    selectedViewName = viewName

    # css classes
    cssActive = u'selected'
    cssInActive = u''

    # menu order weight
    weight = 0

    # sub menu provider name
    subMenuProviderName = None

    def __init__(self, context, request, view, manager):
        super(MenuItem, self).__init__(context, request, view, manager)
        self.view = view
        self.setupFilter()

    def setupFilter(self):
        """Catch location error and set approved attributes.

        Note, this get called before update because the filter method in menu
        manager needs to know that before the menu items update method get
        called.
        """
        try:
            if self.available:
                self.approvedURL = self.url
                self.approved = True
        except TypeError:
            self.approvedURL = None
            self.approved = False

    # override it and use i18n msg ids
    @property
    def title(self):
        return self.i18nTitle or self.__name__

    @property
    def css(self):
        """Return cssActive, cssInActive or None.

        None will force not rendering a HTML attribute in the element tag.
        """
        if self.selected and self.cssActive:
            return self.cssActive
        elif not self.selected and self.cssInActive:
            return self.cssInActive
        else:
            return None

    @property
    def available(self):
        """Available checker call"""
        return True

    @property
    def selected(self):
        """Selected checker call"""
        checker = zope.component.getMultiAdapter((self.context, self.request,
            self.view, self.manager, self), interfaces.ISelectedChecker)
        return checker.selected

    @property
    def url(self):
        return '%s/%s' % (absoluteURL(self.getURLContext(), self.request),
            self.viewName)

    def getURLContext(self):
        return getRoot(self.context)

    def render(self):
        """Return the template with the option 'menus'"""
        return self.template()

    def __repr__(self):
        return '<%s %r>' % (self.__class__.__name__, self.__name__)


class GlobalMenuItem(MenuItem):
    """Global menu item."""

    zope.interface.implements(interfaces.IGlobalMenuItem)


class SiteMenuItem(MenuItem):
    """Site menu item."""

    zope.interface.implements(interfaces.ISiteMenuItem)

    @property
    def available(self):
        """Available checker call"""
        root = zope.proxy.getProxiedObject(getRoot(self.context))
        site = zope.proxy.getProxiedObject(hooks.getSite())
        return site is not root

    def getURLContext(self):
        return hooks.getSite()


class ContextMenuItem(MenuItem):
    """Context menu item."""

    zope.interface.implements(interfaces.IContextMenuItem)

    def getURLContext(self):
        return self.context


class AddMenuItem(MenuItem):
    """Add menu item."""

    zope.interface.implements(interfaces.IAddMenuItem)

    subMenuProviderName = None

    @property
    def selected(self):
        return False

    def getURLContext(self):
        return self.context



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: layer.py 197 2007-04-13 05:03:32Z rineichen $
"""
__docformat__ = "reStructuredText"

import zope.interface
import zope.security
from zope.viewlet import manager

from z3c.menu.ready2go import interfaces


def isAvailable(viewlet):
    try:
        return zope.security.canAccess(viewlet, 'render') and viewlet.approved
    except AttributeError:
        return True


class MenuManager(manager.ConditionalViewletManager):
    """Menu manager for all kind of menu items"""

    zope.interface.implements(interfaces.IMenuManager)

    def filter(self, viewlets):
        """Sort out all viewlets which are explicit not available

        ``viewlets`` is a list of tuples of the form (name, viewlet).
        """
        return [(name, viewlet) for name, viewlet in viewlets
                if isAvailable(viewlet)]


class EmptyMenuManager(object):
    """Empty menu manager."""

    zope.interface.implements(interfaces.IMenuManager)

    def __init__(self, context, request, view):
        self.__updated = False
        self.__parent__ = view
        self.context = context
        self.request = request

    def update(self):
        pass

    def render(self):
        return u''